@@ -375,6 +375,8 @@ class ObjectProxy(six.with_metaclass(_ObjectProxyMetaType)):
     def __iter__(self):
         return iter(self.__wrapped__)
 
+class CallableObjectProxy(ObjectProxy):
+
     def __call__(self, *args, **kwargs):
         return self.__wrapped__(*args, **kwargs)
 
@@ -547,8 +549,8 @@ class FunctionWrapper(_FunctionWrapperBase):
                 enabled, binding)
 
 try:
-    from ._wrappers import (ObjectProxy, FunctionWrapper,
-            BoundFunctionWrapper, _FunctionWrapperBase)
+    from ._wrappers import (ObjectProxy, CallableObjectProxy, FunctionWrapper,
+        BoundFunctionWrapper, _FunctionWrapperBase)
 except ImportError:
     pass
 
@@ -607,16 +609,24 @@ def patch_function_wrapper(module, name):
 
 def transient_function_wrapper(module, name):
     def _decorator(wrapper):
-        @function_wrapper
         def _wrapper(wrapped, instance, args, kwargs):
-            try:
+            target_wrapped = args[0]
+            if instance is None:
+                target_wrapper = wrapper
+            elif inspect.isclass(instance):
+                target_wrapper = wrapper.__get__(None, instance)
+            else:
+                target_wrapper = wrapper.__get__(instance, type(instance))
+            def _execute(wrapped, instance, args, kwargs):
                 (parent, attribute, original) = resolve_path(module, name)
-                replacement = FunctionWrapper(original, wrapper)
+                replacement = FunctionWrapper(original, target_wrapper)
                 setattr(parent, attribute, replacement)
-                return wrapped(*args, **kwargs)
-            finally:
-                setattr(parent, attribute, original)
-        return _wrapper
+                try:
+                    return wrapped(*args, **kwargs)
+                finally:
+                    setattr(parent, attribute, original)
+            return FunctionWrapper(target_wrapped, _execute)
+        return FunctionWrapper(wrapper, _wrapper)
     return _decorator
 
 # A weak function proxy. This will work on instance methods, class
